import os
from flask import flash, redirect, url_for


def make_folder(this_folder):
    rv = True
    if not os.path.exists(os.path.abspath(this_folder)):
        flash('Can create folder!\t' + os.path.abspath(this_folder), 'success')
        rv = True
    else:
        rv_write = eisen_write_test(os.path.abspath(this_folder))
        if not rv_write:
            flash('Folder exists, but write test failed!\t' + os.path.abspath(this_folder), 'danger')
            rv = False
            return rv
        if rv_write:
            rv = True

    if rv:
        try:
            os.makedirs(this_folder)
        except FileExistsError as err:
            if not err.errno == 17:    # pybug, py3.10, makedirs() ignores existing dirs
                print('-make_folder----------- FileExistsError' + str(err.errno))
                rv = False
        except PermissionError:
            print('-make_folder----------- PermissionError')
            rv = False
        except OSError:
            print('-make_folder----------- OSError')
            rv = False
    return rv


def eisen_write_test(write_file):
    test_file = os.path.join(write_file, 'eisen_test.txt')
    try:
        with open(test_file, 'wb') as record_file:
            record_file.write(b'\x03')
        os.remove(test_file)
    except OSError:  # master of desaster
        return False
    return True



# 0.1.2a
##################################################################################
#   MIT License
#
#   Copyright (c) [2021] [René Horn]
#
#   Permission is hereby granted, free of charge, to any person obtaining a copy
#   of this software and associated documentation files (the "Software"), to deal
#   in the Software without restriction, including without limitation the rights
#   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#   copies of the Software, and to permit persons to whom the Software is
#   furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in all
#   copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#   SOFTWARE.
###################################################################################

import io
import os
import queue
import random
##############
import shutil
import threading
import urllib.parse
import urllib.parse
import urllib.request
from concurrent.futures import ThreadPoolExecutor
from time import sleep, strftime, time
from urllib.error import URLError, HTTPError

import urllib3
from eisenradio.api import ghettoApi
import textwrap

# logging.basicConfig(level=logging.DEBUG, format='[%(levelname)s] (%(threadName)-10s) %(message)s',)
# logging.basicConfig(level=logging.INFO, format='[%(levelname)s] (%(threadName)-10s) %(message)s',)

version = '0.8.30'
print(f'{version} ghettorecorder (eisen_radio module)')  # ########## version #############


class GBase:
    # class attribute
    dict_exit = {}  # all "single" started (this time four, each station)
    dict_error = {}

    sleeper = 2  # for exit of all threads
    pool = ThreadPoolExecutor(200)
    radio_base_dir = os.path.dirname(os.path.abspath(__file__)) + '//radiostations'  # if not set set_radio_base_dir()
    settings_path = os.path.dirname(os.path.abspath(__file__)) + '//settings.ini'  # if not set in set_settings_path()
    path = os.getcwd()
    path_to = path + '//'
    timer = 0

    def __init__(self, radio_base_dir=None, settings_path=None):
        self.instance_attr_time = 0
        self.trigger = False
        self.radio_base_dir = radio_base_dir
        self.settings_path = settings_path

    @staticmethod
    def make_directory(str_path):
        access_rights = 0o755
        try:
            os.mkdir(str_path, access_rights)
        except FileExistsError:
            pass
            # print(' Folder exists: ' + str_path)
            return False
        else:
            print('\t--> created directory: ' + str_path)
            return True

    @staticmethod
    def remove_special_chars(str_name):
        # cleanup for writing files and folders

        # my_str = "hey th~!ere. /\ coolleagues?! Straße"
        ret_value = str_name.translate({ord(string): "" for string in '"!@#$%^*()[]{};:,./<>?\|`~=+"""'})
        return ret_value

    @staticmethod
    def this_time():
        time = strftime("%Y-%m-%d %H:%M:%S")
        return time

    def countdown(self, instance_attr_time):
        t = 0
        while not t == instance_attr_time:
            sleep(1)
            self.timer = t
            print(self.timer)
            t += 1
            if t == 0:
                self.trigger = True
        print(f' done {instance_attr_time} {self.trigger}')
        return self.trigger


class GIni(GBase):  # remnant of command line version

    ini_keys = {}  # cls attribute to store selections from ini file, works because of key[key] = value, else not
    srv_param_dict = {}  # all ini keys plus short url, suffix, server type stuff
    start_stop_recording = {}  # ini key: 'start' , 'stop'; while loop check start, working check stop go upper while
    # ini_key + '_single_title', ini_key + '_rec_from_here'
    cost_current_ini = ''  # ini key for cost_dict calc / should be a dict
    cost_dict = {}  # stores len of received headers to calc amount of data searching strings per day
    fail_meta_dict = {}  # can not read metadata from stream, no data
    # list of search strings delimiter blank, first key is named 'STRINGS': Britney Phantom ไม่เคยจะจำ Elton Jim techno
    search_dict = {'STRINGS': 'Britney Spears ไม่เคยจะจำ Elton AC/DC techno Band feat. mix'}  # only show it is working
    list_items = []  # radio key names for display in terminal
    search_title_keys_list = []  # radio short keys, not start recording all streams, only searched titles
    content_type = {}  # header info audio/mpeg

    @staticmethod
    def parse_url_simple_url(radio_url):
        url = radio_url  # whole url is used for connection to radio server

        # 'http:' is first [0], 'ip:port/xxx/yyy' second item [1] in list_url_protocol
        list_url_protocol = url.split("//")
        list_url_ip_port = list_url_protocol[1].split("/")  # 'ip:port' is first item in list_url_ip_port
        radio_simple_url = list_url_protocol[0] + '//' + list_url_ip_port[0]
        return radio_simple_url


class GNet(GBase):
    http_pool = urllib3.PoolManager(num_pools=200)

    @staticmethod
    def load_url(url):
        # returns status code, if server is alive conn.getcode()
        # use urllib, urllib3 causes response to wait "forever" and timeout is not working either
        # print(f' load_url {url}')
        with urllib.request.urlopen(url, timeout=15) as response:
            return response.getcode()

    @staticmethod
    def is_server_alive(url, key):
        # don't delete - urllib3 timeout=5, placebo, retries=None or =2, screw yourself, since half of conn. die
        # we have server up, but content not presented - zombie
        try:
            GNet.load_url(url)
        except HTTPError:  # lots of strange configured web server

            print('HTTPError')
            GBase.dict_error[key] = 'HTTPError - check radio homepage'
            return True
        except URLError as error:  # <urlopen error timed out>
            print(f' ---> {key} server failed: {error} (no recording) {url}')
            GBase.dict_error[key] = error
            print('URLError')
            return False
        return True

    @staticmethod
    def stream_filetype_url(url, key):
        try:
            with urllib.request.urlopen(url, timeout=15) as response:
                headers = response.getheader('Content-Type')
        except Exception as ex:
            print(ex)
            return False

        content_type = ''
        if headers == 'audio/aacp' or headers == 'application/aacp':
            content_type = '.aacp'
        if headers == 'audio/aac':
            content_type = '.aac'
        if headers == 'audio/ogg' or headers == 'application/ogg':
            content_type = '.ogg'
        if headers == 'audio/mpeg':
            content_type = '.mp3'
        if headers == 'audio/x-mpegurl' or headers == 'text/html':
            content_type = '.m3u'
        # application/x-winamp-playlist , audio/scpls , audio/x-scpls ,  audio/x-mpegurl

        try:
            GIni.content_type[key] = headers
            if len(headers) <= 5:
                GIni.content_type[key] = 'audio/mpeg'
            # print(f' stream_filetype_url: {GIni.content_type[key]}')
        except Exception as e:
            print(e)
            pass

        return content_type


class GRecorder:
    unknown_title_name = 'untitled_full_record_'
    path_to_song_dict = {}  # {station : file path}
    current_song_dict = {}  # each thread writes the new title to the station key name {station : title}
    start_write_command = {}  # recorder head thread set command to copy first file
    search_pattern_found = {}

    fifo_web_srv_queue = queue.Queue(maxsize=10)
    fifo_dict = {}
    record_active_dict = {}
    listen_active_dict = {}
    ghetto_measure = {}
    ghettoApi.init_ghetto_measurements(ghetto_measure)

    @staticmethod  # copy keep-alive timeout=3000
    def ghetto_recorder_display_title(url, ini_key):
        # print(f' ghetto_recorder_display_title: {ini_key}')
        update_terminal = ''.encode('utf-8')
        stream_song_name = GRecorder.current_song_dict[ini_key]
        GIni.fail_meta_dict[url] = 'False'  # message display or not
        GRecorder.search_pattern_found[ini_key] = False

        while not GBase.dict_exit[ini_key]:

            if not len(stream_song_name) <= 2:
                #  GRecorder.current_song_dict[ini_key] = stream_song_name  # WRITE SONG NAME   # :DEACTIVATED:
                stream_song_name = GRecorder.current_song_dict[ini_key]  # :REVERSE:

                if not update_terminal == stream_song_name:
                    update_terminal = stream_song_name
                    try:
                        print(f'\t\t {GBase.this_time().split()[1]} title on {ini_key}: \t {update_terminal}')
                        # see what häppens, góod unìt têst case
                    except UnicodeEncodeError:
                        print('\t\t title on ' + ini_key + ':\t' + 'unicode error (check "$ localectl status")')
                        print('\t\t title on ' + ini_key + ':\t' + 'consider to add additional language support.')

                    except Exception as ex:
                        print('\t\t title on ' + ini_key + ':\t' + 'unknown error (print to your terminal)')
                        print(ex)
                    else:
                        pass
            # give recorder command to record, if we got a match from search gui
            #

            try:
                for _ in GIni.search_title_keys_list:
                    if _ == ini_key:
                        # search option was checked,
                        # call - def search_pattern_start_record, search string match?
                        got_it = GRecorder.search_pattern_start_record(stream_song_name, ini_key)
                        if got_it:
                            GIni.start_stop_recording[ini_key] = 'start'
                        else:
                            GIni.start_stop_recording[ini_key] = 'stop'
                            GRecorder.search_pattern_found[ini_key] = False

            except Exception as ex:
                print(ex)

            for sec in range(GBase.sleeper):
                sleep(1)
                if GBase.dict_exit[ini_key]:
                    # print(f' stop get title {ini_key}')
                    break
            # To Do: make (complete) fake headers to keep connection up
            #        sync sleep time with recorder
            #        one file record with name

    @staticmethod
    def search_pattern_start_record(title, ini_key):
        # search title for strings to see if we should start recording this
        search_list = []  # chop the string

        try:
            strings = GIni.search_dict[ini_key]  # dict with user search strings
            search_list = strings.encode('utf-8').lower().split(b' ')
        except KeyError:
            pass

        for search_str in search_list:

            if not len(search_str) <= 1:  # b'' match problem

                if not title.encode('utf-8').lower().find(search_str) == -1:  # find() returns -1, if not found
                    if not GRecorder.search_pattern_found[ini_key]:
                        print(f'<<<< match station: {ini_key} phrase: {search_str}')
                        GRecorder.search_pattern_found[ini_key] = True
                    return True
        return False

    @staticmethod
    def ghetto_recorder_head(directory_save, stream_suffix, radio_short_key):
        # target: tail works without brain until brain call interrupt
        # print(f' ghetto_recorder_head -- {radio_short_key}')

        sleep(GBase.sleeper)
        # reader must work (conn. url) for writing the path from title name, can have a loop here
        fresh_song = 'False'
        first_record = True
        brake_loop = False
        GRecorder.start_write_command[radio_short_key] = False

        stream_song_name = GRecorder.current_song_dict[radio_short_key]

        try:
            GBase.make_directory(GBase.radio_base_dir)
        except FileNotFoundError:
            GRecorder.current_song_dict[radio_short_key] = '[{-_-}] ZZZzz zz z... Recorder Failure!'
            GBase.dict_exit[radio_short_key] = True  # all radio_short_key treads-will-stop
            print('\t---> Recorder Write Failure in : ' + GBase.radio_base_dir)
            return

        GBase.make_directory(GBase.radio_base_dir + '//' + radio_short_key)

        while not brake_loop:
            i = 0
            while not GBase.dict_exit[radio_short_key]:
                if not fresh_song == stream_song_name:

                    if i == 5:
                        print(f' \t\t\t\t\t\t(  (  ( (Eisen Radio) )  )  ) )')
                        i = 0
                    i += 1

                    fresh_song = stream_song_name  # check if it can be removed, we look in a dict now
                    clean_name = GBase.remove_special_chars(stream_song_name)
                    if clean_name == GRecorder.unknown_title_name:  # init name of title on startup
                        clean_name = clean_name + radio_short_key + '_date_' + GBase.this_time()
                        clean_name = GBase.remove_special_chars(clean_name)

                    if first_record:
                        fresh_file_path = directory_save + '//' + '_incomplete_' + str(clean_name) + str(stream_suffix)
                        brake_loop = True
                        GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path
                        GRecorder.start_write_command[radio_short_key] = True

                        first_record = False
                    else:
                        fresh_file_path = directory_save + '//' + clean_name + stream_suffix
                        GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path  # NEW PATH ... ... ...

                    if GBase.dict_exit[radio_short_key]:
                        # print(f' exit head__ {radio_short_key}')
                        break

                    while not GBase.dict_exit[radio_short_key]:

                        stream_song_name = GRecorder.current_song_dict[radio_short_key]
                        if not fresh_song == stream_song_name:
                            break

                        for sec in range(5):
                            if GBase.dict_exit[radio_short_key]:
                                fresh_file_path = directory_save + '//' + '_incomplete_' + clean_name + stream_suffix
                                GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path
                                # print(f' exit head_ {radio_short_key}')
                                break
                            sleep(1)

    @staticmethod  # org
    def ghetto_recorder_tail(url, key, path_to_save, suffix):
        # shall be as dump as possible

        while not GBase.dict_exit[key]:
            try:
                if GRecorder.start_write_command[key]:
                    break
            except KeyError:
                pass
            else:
                sleep(.1)

        stream_request_size = io.DEFAULT_BUFFER_SIZE
        old_time = time()
        request = ''
        rand_short = random.randrange(4, 13, 3)

        ghetto_recorder = path_to_save + '//__ghetto_recorder' + str(suffix)
        try:
            ghetto_copy = GRecorder.path_to_song_dict[key]
        except KeyError:
            GRecorder.current_song_dict[key] = '[{-_-}] ZZZzz zz z... Recorder Failure!'
            GBase.dict_exit[key] = True  # all key treads-will-stop
            return
        try:
            request = GNet.http_pool.request('GET', url,
                                             headers={'Connection': 'keep-alive'},
                                             timeout=3000,
                                             preload_content=False)
        except Exception as e:
            print(e)
            pass

        while not GBase.dict_exit[key]:
            with open(ghetto_recorder, 'wb') as record_file:

                for chunk in request.stream(stream_request_size):  # chunks to file

                    # ############################################################## rec
                    # print(f'rec {GRecorder.record_active_dict[key]}')
                    # print(f'lis {GRecorder.listen_active_dict[key]}')
                    if GRecorder.record_active_dict[key]:
                        record_file.write(chunk)

                    # ############################################################## rec
                    if not ghetto_copy == GRecorder.path_to_song_dict[key]:

                        try:
                            if os.path.exists(ghetto_copy):
                                os.remove(ghetto_copy)
                            record_file.flush()

                            ghetto_size = os.path.getsize(ghetto_recorder)
                            if int(ghetto_size) >= int(stream_request_size):
                                shutil.copyfile(ghetto_recorder, ghetto_copy)
                        except Exception as ex:
                            print(ex)
                        else:
                            record_file.truncate()
                            record_file.seek(0)

                    ghetto_copy = GRecorder.path_to_song_dict[key]  # SET NEW PATH (title) after copy
                    # #################################
                    if time() - old_time > rand_short:
                        old_time = time()
                        rh = request.headers
                        rand_short = random.randrange(4, 12, 2)

                    if not chunk:
                        request = GNet.http_pool.request('GET', url,
                                                         headers={'Connection': 'keep-alive'},
                                                         timeout=3000,
                                                         preload_content=False)
                        rh = request.headers  # into oblivion
                        break
                    if GBase.dict_exit[key]:
                        sleep(2)  # wait for last file name _incomplete_.....
                        try:
                            record_file.flush()

                            ghetto_size = os.path.getsize(ghetto_recorder)
                            # print(f' ghetto_recorder: {int(ghetto_size)}')
                            # print(f' stream_request_size): {int(stream_request_size)}')

                            if int(ghetto_size) >= int(stream_request_size):
                                ghetto_copy = GRecorder.path_to_song_dict[key]  # full path last file name
                                print(f'\t "{key}" last file (marked: _incomplete_): '
                                      f'{GRecorder.current_song_dict[key]} ')

                                shutil.copyfile(ghetto_recorder, ghetto_copy)
                            record_file.close()

                            if os.path.exists(ghetto_recorder):
                                os.remove(ghetto_recorder)
                        except UnicodeEncodeError:
                            print(f' {key} copy last file (marked: _incomplete_):')
                        except Exception as ex:
                            print(ex)
                        break
            if GBase.dict_exit[key]:
                # print(f' exit file record {key}')
                break

    @staticmethod
    def get_metadata_from_stream_loop(url, key):
        # urllib3 request.stream with headers={'Icy-MetaData': '1'} , keep open (mdr, swr, br5) test

        response = ''
        request = ''
        title = ''
        metadata_content = ''
        old_time = time()
        rand_short = random.randrange(4, 12, 2)
        rand_long = random.randrange(50, 80, 1)

        while not GBase.dict_exit[key]:
            try:
                response = GNet.http_pool.request('GET', url,
                                                  headers={'Connection': 'keep-alive'},
                                                  timeout=3000, preload_content=False)
            except Exception as e:
                print(e)
                pass

            while not GBase.dict_exit[key]:

                if time() - old_time > rand_short:
                    old_time = time()
                    rh = response.headers  # keep conn. alive
                    rand_short = random.randrange(4, 12, 2)
                    # print(response.headers)

                if time() - old_time > rand_long:  # new connection
                    old_time = time()
                    rand_long = random.randrange(50, 80, 1)
                    break
                try:
                    start_time = time()
                    request = GNet.http_pool.request('GET', url,
                                                     headers={'Icy-MetaData': '1'},
                                                     preload_content=False)
                    try:
                        GRecorder.ghetto_measure[key + ',request_time'] = round((time() - start_time) * 1000)
                        GRecorder.ghetto_measure[key + ',suffix'] = request.headers['content-type']
                    except KeyError:
                        pass
                    try:
                        icy_genre = request.headers["icy-genre"]  # [0:30]
                        icy_name = request.headers["icy-name"]
                        icy_br = request.headers["icy-br"]
                        GRecorder.ghetto_measure[key + ",icy_genre"] = icy_genre
                        GRecorder.ghetto_measure[key + ",icy_name"] = icy_name
                        GRecorder.ghetto_measure[key + ",icy_br"] = icy_br
                    except KeyError:
                        pass
                    # print(request.headers)
                except Exception as e:
                    print(e)
                    pass

                try:
                    pass

                except KeyError:
                    pass
                try:
                    icy_metadata = request.headers['icy-metaint']
                    icy_metadata = int(icy_metadata)

                    request.read(icy_metadata)
                    chunk_1b = request.read(1)
                    chunk_1b = ord(chunk_1b)
                    read_bytes = chunk_1b * 16
                    read_bytes = int(read_bytes)
                    metadata_content = request.read(read_bytes)
                except KeyError:
                    pass
                try:
                    metadata_content = metadata_content.decode('utf-8')
                    title_info = metadata_content.split(";")
                    title_info = title_info[0].split("=")
                    title = str(title_info[1])
                    title = GBase.remove_special_chars(title)

                    request.release_conn()  # REQUEST one shot anyway
                    # print(f' -> {key} + {title}')
                except IndexError:
                    try:
                        metadata_content = metadata_content
                        title_info = metadata_content.split(";")
                        title_info = title_info[0].split("=")
                        title = str(title_info[1])
                        title = GBase.remove_special_chars(title)
                    except Exception:
                        request.release_conn()  # REQUEST one shot anyway
                        pass

                else:
                    try:
                        if title[0] == "'" and title[-1] == "'":
                            GRecorder.current_song_dict[key] = title[1:-1]
                        else:
                            GRecorder.current_song_dict[key] = title
                        # print(f' -> {key} + {GRecorder.current_song_dict[key]}')
                    except KeyError:
                        pass
                    except Exception as e:
                        print(e)
                        pass
                    else:
                        for sec in range(rand_short):
                            if GBase.dict_exit[key]:
                                # print(f' out reader {key}')
                                break
                            sleep(1)

    @staticmethod
    def playlist_m3u(url, key):
        # returns the first server of the playlist (not only m3u)
        try:
            read_url = GNet.http_pool.request('GET', url, preload_content=False)
        except Exception as ex:
            print(ex)
        else:
            file = read_url.read().decode('utf-8')

            m3u_lines = file.split("\n")
            # print(' \n    m3u_lines    ' + file)
            m3u_lines = list(filter(None, m3u_lines))  # remove empty rows
            m3u_streams = []
            for row_url in m3u_lines:
                if row_url[0:4].lower() == 'http'.lower():
                    m3u_streams.append(row_url)  # not to lower :)
                    # print(len(m3u_streams))

            if len(m3u_streams) > 1:
                print(f' {key} Have more than one server in playlist_m3u. !!! Take first stream available.')
                play_server = m3u_streams[0]
                return play_server
            if len(m3u_streams) == 1:
                # print(' One server found in playlist_m3u')
                play_server = m3u_streams[0]
                return play_server
            if len(m3u_streams) == 0:
                # print(' No http ... server found in playlist_m3u !!! -EXIT-')
                return False


def check_alive_playlist_container(str_key, str_url):
    # playlist url?
    if str_url[-4:] == '.m3u' or str_url[-4:] == '.pls':  # or url[-5:] == '.m3u8' or url[-5:] == '.xspf':
        # take first from the list
        is_playlist_server = GRecorder.playlist_m3u(str_url, str_key)

        if not is_playlist_server == '':
            if GNet.is_server_alive(str_url, str_key):
                str_url = is_playlist_server
            else:
                print('   --> playlist_server server failed, no recording')
            return str_url
    else:
        GNet.is_server_alive(str_url, str_key)
        return False


def record(ini_key, url):
    stream_suffix = GNet.stream_filetype_url(url, ini_key)  # ret and write in content-type dict
    GRecorder.current_song_dict[ini_key] = GRecorder.unknown_title_name
    GIni.start_stop_recording[ini_key] = 'start'
    # GIni.start_stop_recording[ini_key] = 'stop'            # init it here, should be set via user interface
    GIni.start_stop_recording[ini_key + '_adv'] = 'start_from_here'  # MUST be set or a key error in record def:

    dir_save = GBase.radio_base_dir + '//' + ini_key

    threading.Thread(target=GRecorder.ghetto_recorder_display_title,
                     args=(url, ini_key),
                     daemon=True).start()
    threading.Thread(target=GRecorder.ghetto_recorder_head,
                     args=(dir_save, stream_suffix, ini_key),
                     daemon=True).start()
    threading.Thread(target=GRecorder.ghetto_recorder_tail,
                     args=(url, ini_key, dir_save, stream_suffix),
                     daemon=True).start()
    threading.Thread(target=GRecorder.get_metadata_from_stream_loop,
                     args=(url, ini_key),
                     daemon=True).start()

    # ################################## end ########################################
    # this version ends here. no loop



import certifi
from os import path, environ
from flask import Flask
from eisenradio.instance.config_apfac import write_config, remove_config
from eisenradio.api import api

# android ssl fix
environ['SSL_CERT_FILE'] = certifi.where()
script_path = path.dirname(__file__)

""" Flask application factory with blueprints  """

""" 'Home' and 'Util' load modules, templates, styles, favicon from its own project folders """


def create_app(test=None):    # must be None

    app = Flask(__name__,
                instance_path=path.join(script_path, 'instance'))

    with app.app_context():

        api.init_app(app)

        if test is not None:
            """------------- TEST RUN -------------------
            exec  write_config('test') at first from pytest"""
            write_config('test')
            # remove in teardown

        if not test:
            # environ['WERKZEUG_RUN_MAIN'] = 'true'
            is_snap_device = 'SNAP' in environ  # write in [SNAP_USER_COMMON]
            is_android_device = 'ANDROID_STORAGE' in environ

            if not is_snap_device and not is_android_device:

                write_config('prod')
                remove_config()

            if is_snap_device:

                write_config('snap')
                remove_config(environ["SNAP_USER_COMMON"])

            if is_android_device:

                write_config('android')
                # db must exist, no loader, write os and flask env from module, fail in create_app

        # helper stuff
        from eisenradio.lib.platform_helper import main as start_frontend
        from eisenradio.lib.eisdb import install_new_db as create_install_db

        from eisenradio.eisenhome import routes as home_routes
        from eisenradio.eisenutil import routes as util_routes

        # Register Blueprints (pointer to parts of the application, subprojects in production)
        app.register_blueprint(home_routes.eisenhome_bp)
        app.register_blueprint(util_routes.eisenutil_bp)

        create_install_db(app.config['DATABASE'])

        if test:
            print(f"\tapp.config:       {app.config}\n")
        if not test:
            start_frontend()
        return app




import os
from flask import Blueprint, render_template, request, url_for, flash, redirect, make_response, jsonify
import eisenradio.lib.ghetto_recorder as ghetto
from eisenradio.lib.eisdb import get_db_connection, status_read_status_set, render_picture, get_download_dir, get_post
from eisenradio.eisenutil.eisutil import make_folder
from eisenradio.eisenutil.measure import measure_meta, stopped_stations
# Blueprint Configuration
eisenutil_bp = Blueprint(
    'eisenutil_bp', __name__,
    template_folder='bp_util_templates',
    static_folder='bp_util_static',
    static_url_path='/bp_util_static'
)


@eisenutil_bp.route('/about', methods=('GET', 'POST'))
def about():
    conn = get_db_connection()
    posts = conn.execute('SELECT * FROM posts').fetchall()
    try:
        download_path = get_download_dir()
    finally:
        conn.close()

    if request.method == 'POST':
        if request.form['browser']:
            status_read_status_set(True, 'eisen_intern', 'browser_open', '1')
            return redirect(url_for('eisenutil_bp.about'))

    is_browser_on = status_read_status_set(False, 'eisen_intern', 'browser_open', '1')
    return render_template('bp_util_about.html',
                           is_browser_on=is_browser_on,
                           download_path=download_path)


@eisenutil_bp.route('/create', methods=('GET', 'POST'))
def create():
    if request.method == 'POST':
        rv_req = request.form['title']
        title = rv_req.replace(" ", "")
        content = request.form['content']

        if not title:
            flash('Name is required!', 'warning')
        if not content:
            flash('URL is required!', 'warning')

        # Image
        if request.files['inputFile']:
            file = request.files['inputFile']
            content_type = file.content_type
            print(f' name {file.name} content-type {file.content_type}')
            try:
                db_img = file.read()
                image = render_picture(db_img, 'encode')
            except Exception as e:
                print(e)
                image = None
                content_type = None
                pass
        else:
            image = None
            content_type = None

        if request.form['text']:
            text = request.form['text']
        else:
            text = None

        conn = get_db_connection()
        posts = conn.execute('SELECT * FROM posts').fetchall()
        # if we later want extra save folders for each radio
        try:
            request_path = posts[0]["download_path"]
        except Exception as e:
            print(e)
            print(' looks like the first radio to create, no save to path set')
            conn.execute('INSERT INTO posts (title, content) VALUES (?, ?)',
                         (title, content))
        else:
            conn.execute('INSERT INTO posts (title, content, download_path, pic_data, pic_content_type, pic_comment) '
                         'VALUES (?, ?, ?, ?, ?, ?)',
                         (title, content, request_path, image, content_type, text))

        conn.commit()
        conn.close()
        return redirect(url_for('eisenhome_bp.index'))

    return render_template('bp_util_create.html')


@eisenutil_bp.route('/save', methods=('GET', 'POST'))
def save():
    rv = None
    if request.method == 'GET':
        """fail if db empty"""
        try:
            rv = os.path.abspath(get_download_dir())
        except TypeError:
            return render_template('bp_util_save.html', save_to='no-directory-found')
        return render_template('bp_util_save.html', save_to=os.path.abspath(get_download_dir()))

    if request.method == 'POST':
        request_path = os.path.abspath(request.form['Path_Save_To'])

        if not request_path:
            flash('A folder is required!', 'warning')
            try:
                rv = os.path.abspath(get_download_dir())
            except TypeError:
                return render_template('bp_util_save.html', save_to='no-directory-found')
            return render_template('bp_util_save.html', save_to=os.path.abspath(get_download_dir()))

        if request_path:
            made_it = make_folder(request_path)
            if not made_it:
                flash('No folder created! exists or denied', 'warning')
                return render_template('bp_util_save.html', save_to=os.path.abspath(get_download_dir()))

            conn = get_db_connection()
            records = conn.execute('select id from posts').fetchall()
            if not records:
                flash('Noting in Database!', 'warning')
                try:
                    return render_template('bp_util_save.html', save_to=os.path.abspath(get_download_dir()))
                except TypeError:
                    return render_template('bp_util_save.html', save_to='no-row-in-table-create-a-radio')
            for id_num in records:
                # print("Radio Id:", id_num[0])
                conn.execute('UPDATE posts SET download_path = ? WHERE id = ?', (request_path, id_num[0]))
            conn.commit()
            conn.close()

            flash(('Save files to: ' + request_path), "success")
            return redirect(url_for('eisenhome_bp.index'))

        return render_template('bp_util_save.html', save_to=os.path.abspath(get_download_dir()))


@eisenutil_bp.route('/<int:id>/edit', methods=('GET', 'POST'))
def edit(id):
    no_image = False
    post = get_post(id)

    if request.method == 'POST':
        rv_req = request.form['title']
        title = rv_req.replace(" ", "")
        content = request.form['content']

        if not title:
            flash('Title is required!', 'warning')
        # Image, if empty keep db entry as it is
        if request.files['inputFile']:
            file = request.files['inputFile']
            content_type = file.content_type
            print(f' name {file.name} content-type {file.content_type}')
            try:
                db_img = file.read()
                image = render_picture(db_img, 'encode')
            except Exception as e:
                print(e)
                image = None
                content_type = None
                no_image = True
                pass
        else:
            image = None
            content_type = None
            no_image = True

        if request.form['text']:
            text = request.form['text']
            if text == 'None':
                text = ' '
        else:
            text = ' '

        conn = get_db_connection()
        if image:
            conn.execute('UPDATE posts SET title = ?, content = ?, pic_data = ?, pic_content_type = ?, '
                         'pic_comment = ? WHERE id = ?',
                         (title, content, image, content_type, text, id))
        else:
            conn.execute('UPDATE posts SET title = ?, content = ?, pic_comment = ?  WHERE id = ?',
                         (title, content, text, id))

        conn.commit()
        conn.close()
        return redirect(url_for('eisenhome_bp.index'))

    return render_template('bp_util_edit.html', post=post)


@eisenutil_bp.route('/<int:post_id>')
def post(post_id):
    post = get_post(post_id)
    url_port = ghetto.GIni.parse_url_simple_url(post["content"])

    return render_template('bp_util_post.html',
                           post=post,
                           url_port=url_port)


@eisenutil_bp.route('/sw')
def stream_watcher():
    resp = make_response("stream watcher    (ಠ_ಠ)")
    return resp


@eisenutil_bp.route('/cache_info', methods=['GET'])
def cache_info():
    json_lists = measure_meta()
    if len(json_lists) == 0:
        json_lists = '-empty-'
    return jsonify({"cache_result": json_lists})


@eisenutil_bp.route('/delete_info', methods=['GET'])
def delete_info():
    json_list = stopped_stations()
    if len(json_list) == 0:
        json_list = '-empty-'
    return jsonify({"stopped_result": json_list})



import sys
from os import path
from eisenradio import create_app

this_dir = path.abspath(path.join(path.dirname("__file__")))
api_dir = path.abspath(path.join(path.dirname("__file__"), 'eisenradio/api'))
home_dir = path.abspath(path.join(path.dirname("__file__"), 'eisenradio/eisenhome'))
utils_dir = path.abspath(path.join(path.dirname("__file__"), 'eisenradio/eisenutils'))
lib_dir = path.abspath(path.join(path.dirname("__file__"), 'eisenradio/lib'))
sys.path.append(path.abspath(this_dir))
sys.path.append(path.abspath(api_dir))
sys.path.append(path.abspath(home_dir))
sys.path.append(path.abspath(utils_dir))
sys.path.append(path.abspath(lib_dir))

app = create_app()

if __name__ == "__main__":
    app.run(host='localhost', port=5050)



class Api:
    def __init__(self):
        self.config = {}

    def init_app(self, app):
        self.config = app.config


class GhettoApi:
    def __init__(self):
        self.lis_btn_dict = {}
        self.rec_btn_dict = {}
        self.radios_in_view_dict = {}
        self.ghetto_measure_dict = {}

    def init_lis_btn_dict(self, lis_btn_dict):
        self.lis_btn_dict = lis_btn_dict

    def init_rec_btn_dict(self, rec_btn_dict):
        self.rec_btn_dict = rec_btn_dict

    def init_radios_in_view(self, radios_in_view_dict):
        self.radios_in_view_dict = radios_in_view_dict

    def init_ghetto_measurements(self, ghetto_measure_dict):
        self.ghetto_measure_dict = ghetto_measure_dict


api = Api()
ghettoApi = GhettoApi()



from os import path, remove, environ
from dotenv import load_dotenv

try:
    from flask import current_app
except ImportError:
    pass

this_script_dir = path.dirname(__file__)


def write_config(conf, empty_db=None):
    global this_script_dir
    # get root of app, rip off one subdir .go up
    app_root = path.dirname(this_script_dir)

    radio_db_dir = path.join(app_root, 'app_writeable', 'db')

    # SECRET_KEY
    secret_num = '9fc2e5bd8372430fb6a1012af0b51f37'
    # PRODUCTION, DEV
    db_path_prod = path.join(radio_db_dir, 'database.db')
    # TEST
    db_path_test = path.join(radio_db_dir, 'eisenradio_test.db')
    # SNAP
    db_path_snap = ''
    if conf == 'snap':
        db_path_snap = environ["SNAP_USER_COMMON"] + '//database.db'
    db_path_android = ''
    if conf == 'android':
        db_path_android = path.abspath(path.join(app_root, 'app_writeable', 'db', 'eisenradio_android.db'))

    if empty_db is None:
        empty_db = 'False'
    if empty_db == 'True':
        empty_db = 'True'

    list_prod = [
        'DATABASE=' + db_path_prod,
        'FLASK_ENV=production',
        'DEBUG=False',
        'TESTING=False',
        'SECRET_KEY=' + secret_num,
        'EMPTY_DB=' + empty_db
    ]

    list_test = [
        'DATABASE=' + db_path_test,
        'FLASK_ENV=development',
        'DEBUG=True',
        'TESTING=True',
        'SECRET_KEY=' + secret_num,
        'EMPTY_DB=' + empty_db
    ]

    list_dev = [
        'DATABASE=' + db_path_prod,
        'FLASK_ENV=development',
        'DEBUG=True',
        'TESTING=True',
        'SECRET_KEY=' + secret_num,
        'EMPTY_DB=' + empty_db
    ]

    list_snap = [
        'DATABASE=' + db_path_snap,
        'FLASK_ENV=production',
        'DEBUG=False',
        'TESTING=False',
        'SECRET_KEY=' + secret_num,
        'EMPTY_DB=' + empty_db
    ]

    dict_android_os_env = {"DATABASE": str(db_path_android),
                           "FLASK_ENV": 'production',
                           "DEBUG": 'False',
                           "TESTING": 'False',
                           "SECRET_KEY": 'environ[] = ' + str(secret_num),
                           "EMPTY_DB": str(empty_db),
                           "ME_TOO": 'False'
                           }

    """loader is not needed if os env loaded directly here in /instance folder (but can load from share)"""
    loader = ''
    db = ''
    f_env = ''
    debug = ''
    testing = ''

    if conf == 'prod':
        loader = list_prod  # write to .env
        db = db_path_prod  # make flask env, config.update
        f_env = 'production'
        debug = 'False'
        testing = 'False'
    if conf == 'test':
        loader = list_test
        db = db_path_test
        f_env = 'development'
        debug = 'True'
        testing = 'True'
    if conf == 'dev':
        loader = list_dev
        db = db_path_prod
        f_env = 'development'
        debug = 'True'
        testing = 'True'
    if conf == 'snap':
        loader = list_snap
        if db_path_snap == '':
            raise ValueError('\n SNAP got no db path!!')
        db = db_path_snap
        f_env = 'production'
        debug = 'False'
        testing = 'False'
    if conf == 'android':
        loader = ''
        if db_path_android == '':
            raise ValueError('\n Android got no db path!!')
        db = db_path_android
        f_env = 'production'
        debug = 'False'
        testing = 'False'

    if not conf == 'snap' and not conf == 'android':
        remove_config()
        with open(path.join(this_script_dir, '.env'), 'w') as writer:
            for line in loader:
                writer.write(line + '\n')
            writer.flush()
        load_config_os()

    if conf == 'snap':
        db_path_snap_dir = environ["SNAP_USER_COMMON"]
        remove_config(db_path_snap_dir)
        with open(path.join(db_path_snap_dir, '.env'), 'w') as writer:
            for line in loader:
                writer.write(line + '\n')
            writer.flush()
        load_config_os(db_path_snap_dir)

    if conf == 'android':
        for var, setter in dict_android_os_env.items():
            environ[var] = setter

    try:
        current_app.config.update(SECRET_KEY=secret_num,
                                  DATABASE=path.abspath(db),
                                  FLASK_ENV=f_env,
                                  DEBUG=debug,
                                  TESTING=testing,
                                  )
    except RuntimeError:
        """test from outside can not connect to flask 'current_app' """
        pass


def load_config_os(new_path=None):
    global this_script_dir
    if new_path is not None:
        this_script_dir = new_path
    load_dotenv(path.join(this_script_dir, '.env'))


def remove_config(new_path=None):
    global this_script_dir
    if new_path is not None:
        this_script_dir = new_path
    try:
        remove(path.join(this_script_dir, '.env'))
    except OSError:
        pass



import os
import threading
from time import sleep
from flask import jsonify, request, flash, redirect, url_for

import eisenradio.lib.ghetto_recorder as ghetto
from eisenradio.lib.eisdb import get_db_connection, status_read_status_set, get_download_dir
from eisenradio.api import ghettoApi


first_run_index = True
listen_btn = False

status_listen_btn_dict = {}  # {table id 15: 1 } on
status_record_btn_dict = {}
radios_in_view_dict = {}
ghettoApi.init_lis_btn_dict(status_listen_btn_dict)
ghettoApi.init_rec_btn_dict(status_record_btn_dict)
ghettoApi.init_radios_in_view(radios_in_view_dict)

active_streamer_dict = {}

last_btn_id = None  # current button pressed, or btn predecessor / to redraw, if next is pressed / first is null->err
listen_last_url = None
combo_master_timer = 0
progress_master_percent = 0

active_displays = {}
eisen_threads = []


def enum_radios():
    conn = get_db_connection()
    posts = conn.execute('SELECT * FROM posts').fetchall()
    conn.close()
    return posts


def index_first_run(posts):
    global first_run_index
    if first_run_index:
        first_run_index = False

        ghetto.GBase.pool.submit(progress_timer)

        for row in posts:
            # init btn not pressed
            status_listen_btn_dict[row['id']] = 0
            status_record_btn_dict[row['id']] = 0
            # api
            radios_in_view_dict[row['id']] = status_read_status_set(False, 'posts', 'title', row['id'])


def curr_radio_listen():
    current_station = ''
    current_table_id = ''
    for key_table_id, val in status_listen_btn_dict.items():
        if val:
            current_station = status_read_status_set(False, 'posts', 'title', key_table_id)
            current_table_id = key_table_id
    return current_station, current_table_id


def index_posts_clicked(post_request):
    # this version does not have a relay server
    global listen_btn
    global last_btn_id
    global listen_last_url

    try:
        button_list = post_request['name'].split('_')
    except KeyError:
        return
    table_id = button_list[1]
    button = button_list[0]
    current_station = ''

    if button == 'Listen':  # turn values, update button status dict

        try:
            # new radio not yet in dict, deactivate till restart, buttons have no function
            rv = status_listen_btn_dict[int(table_id)]
            rv_station_lis = status_read_status_set(False, 'posts', 'title', table_id)
        except KeyError:
            return False
        if status_listen_btn_dict[int(table_id)]:  # button was down
            status_listen_btn_dict[int(table_id)] = 0
            dispatch_master(int(table_id), 'Listen', 0)
            if not last_btn_id == table_id:
                return jsonify({'result': 'auto_click, no_audio_action', 'table_ident': current_station,
                                'radio_table_id': table_id, 'class_val': post_request['class_val']})
        else:
            status_listen_btn_dict[int(table_id)] = 1  # button just pressed, register state, new station

            if not last_btn_id == table_id:  # other btn pressed
                if listen_btn:  # was playing before actual btn
                    switch_btn = last_btn_id  # press abandoned btn, play new url
                    last_btn_id = table_id
                    listen_last_url = query_radio_url(table_id)  # radio url
                    dispatch_master(int(table_id), 'Listen', 1)
                    current_station = status_read_status_set(False, 'posts', 'title', table_id)
                    # trigger auto_click
                    return jsonify(
                        {'former_button_to_switch': switch_btn,
                         'query_url': listen_last_url,
                         'if not last_btn_id == table_id': '------ last_btn_id --------',
                         'result': 'activate_audio', 'table_ident': current_station, 'radio_table_id': table_id,
                         'class_val': post_request['class_val']})

            dispatch_master(int(table_id), 'Listen', 1)

        listen_last_url = query_radio_url(table_id)  # radio url
        last_btn_id = table_id

        listen_btn = False
        for val in status_listen_btn_dict:
            if status_listen_btn_dict[val]:
                listen_btn = True
                break

        if listen_btn:
            current_station = status_read_status_set(False, 'posts', 'title', table_id)
            return jsonify(
                {'result': 'activate_audio', 'former_button_to_switch': False, 'buttons': button,
                 'query_url': listen_last_url,
                 'table_ident': current_station, 'radio_table_id': table_id, 'class_val': post_request['class_val']})
        else:
            listen_last_url = ''
            return jsonify(
                {'result': 'deactivate_audio', 'former_button_to_switch': False, 'buttons': button,
                 'query_url': listen_last_url,
                 'table_ident': current_station, 'radio_table_id': table_id, 'class_val': post_request['class_val']})

    if button == 'Record':
        try:
            rv = status_record_btn_dict[int(table_id)]
        except KeyError:
            return False

        conn = get_db_connection()
        rec_station = status_read_status_set(False, 'posts', 'title', table_id)
        conn.close()

        if status_record_btn_dict[int(table_id)]:
            del active_streamer_dict[rec_station]
            status_record_btn_dict[int(table_id)] = 0
            dispatch_master(int(table_id), 'Record', 0)
        else:
            active_streamer_dict[rec_station] = str(table_id)
            status_record_btn_dict[int(table_id)] = 1
            dispatch_master(int(table_id), 'Record', 1)

    """make combo box with anchor jumper"""
    json_str = ''
    for streamer_name, streamer_id in active_streamer_dict.items():
        json_str = json_str + str(streamer_name) + '=' + str(streamer_id) + ','
    if not json_str:
        json_str = str('empty_json')

    return jsonify(
        {'result': 'no_audio_result',
         'buttons': button, 'rec_btn_id': table_id, 'streamer': json_str,
         'former_button_to_switch': False,
         'query_url': listen_last_url,
         'radio_table_id': table_id, 'class_val': post_request['class_val']})


def query_radio_url(table_id):
    str_url = status_read_status_set(
        False, 'posts', 'content', table_id)
    return str_url


def set_radio_path(table_id):
    radio_path = status_read_status_set(
        False, 'posts', 'download_path', table_id)
    ghetto.GBase.radio_base_dir = radio_path


def dispatch_master(table_id, button, status):
    set_radio_path(table_id)
    str_radio = status_read_status_set(
        False, 'posts', 'title', table_id)

    if button == 'Listen':
        dispatch_listen_btn(str_radio, table_id, status)

    elif button == 'Record':
        dispatch_record_btn(str_radio, table_id, status)


def dispatch_listen_btn(str_radio, table_id, listen_status):

    if listen_status:
        ghetto.GRecorder.listen_active_dict[str_radio] = True  # later needed for relay

        if not status_record_btn_dict[table_id]:  # recorder not started / index_posts_ajax()
            ghetto.GRecorder.record_active_dict[str_radio] = False  # pass record write, else key error (to fix)
            dispatch_record_title(table_id, False, True)  # start title grabber

    if not listen_status:
        ghetto.GRecorder.listen_active_dict[str_radio] = False
        if not status_record_btn_dict[table_id]:
            dispatch_recorder_stop(table_id)


def dispatch_record_btn(str_radio, table_id, record_status):

    if record_status:
        ghetto.GRecorder.record_active_dict[str_radio] = True

        if not status_listen_btn_dict[table_id]:  # listen not started / index_posts_ajax()
            ghetto.GRecorder.listen_active_dict[str_radio] = False
        dispatch_record_title(table_id, True, False)

    else:
        ghetto.GRecorder.record_active_dict[str_radio] = False
        dispatch_recorder_stop(table_id)


def dispatch_recorder_stop(table_id):

    str_radio = status_read_status_set(
        False, 'posts', 'title', table_id)
    # stop recorder threads
    ghetto.GBase.dict_exit[str_radio] = True
    try:
        del active_displays[table_id]
        sleep(2)
        if status_listen_btn_dict[table_id]:
            dispatch_record_title(table_id, False, True)
    except KeyError:
        pass


def dispatch_record_title(table_id, record, title):

    str_radio = status_read_status_set(
        False, 'posts', 'title', table_id)
    str_url = status_read_status_set(
        False, 'posts', 'content', table_id)

    ghetto.GBase.dict_exit[str_radio] = False
    # test for alive, container docker or snap, and playlist / data_base_auto
    str_checked_url_m3u = ghetto.check_alive_playlist_container(str_radio, str_url)
    if str_checked_url_m3u:
        # replace m3u with real url in database
        conn = get_db_connection()
        print('UPDATE posts SET content = ? WHERE id = ?', (str(str_checked_url_m3u), str(table_id)))
        conn.execute('UPDATE posts SET content = ? WHERE id = ?', (str(str_checked_url_m3u), str(table_id)))
        conn.commit()
        conn.close()
        str_url = str_checked_url_m3u
    # start threads
    try:
        print(ghetto.GBase.dict_error[str_radio])  # show connection error in display
    except KeyError:
        # no error, no key exists

        if title:
            thread = threading.Thread(target=ghetto.GRecorder.get_metadata_from_stream_loop, args=(str_url, str_radio),
                                      daemon=True)
            # thread.setDaemon(True)  # non block
            eisen_threads.append(thread)
            thread.start()

        elif record:
            # ghetto.record(str_radio, str_url)

            thread = threading.Thread(name=table_id, target=ghetto.record, args=(str_radio, str_url), daemon=True)
            # thread.setDaemon(True)
            eisen_threads.append(thread)
            thread.start()

        thread = threading.Thread(target=display_write_db, args=(str_radio, table_id), daemon=True)
        # thread.setDaemon(True)  # non block
        eisen_threads.append(thread)
        thread.start()
        return True
    else:
        # show error on display
        active_displays[table_id] = ghetto.GBase.dict_error[str_radio]
        ghetto.GBase.dict_exit[str_radio] = True
        return False


def display_write_dict():

    conn = get_db_connection()
    try:
        titles = conn.execute('SELECT id, display FROM posts').fetchall()
        for row in titles:
            table_id = row[0]
            title = row[1]
            if title is not None:
                active_displays[table_id] = title
    except KeyError:
        pass
    conn.close()


def display_write_db(radio, table_id):  # title to db for each radio; other thread can collect or pull it or else
    title = ''
    while not ghetto.GBase.dict_exit[radio]:

        try:
            title = ghetto.GRecorder.current_song_dict[radio]
        except KeyError:
            pass
        display_update_table(title, table_id)
        try:
            display_update_table(title, table_id)
        except Exception as e:
            print(e)
            pass

        for sec in range(5):
            sleep(1)
            if ghetto.GBase.dict_exit[radio]:
                title = 'Null'
                display_update_table(title, table_id)
                break


def display_update_table(title, table_id):

    conn = get_db_connection()
    try:
        conn.execute('UPDATE posts SET display = ? WHERE id = ? ;', (str(title), str(table_id)))   # parameter tuple
    except Exception as e:
        print(f' utf char prob. check lang. support display_update_table(): {e}')
    else:
        conn.commit()
        conn.close()
        display_write_dict()


def display_clean_titles():
    conn = get_db_connection()
    records = conn.execute('select id from posts').fetchall()
    for id_num in records:
        conn.execute('UPDATE posts SET display = ? WHERE id = ?', (None, id_num[0]))
    conn.commit()
    conn.close()


def progress_timer():

    global combo_master_timer   # combo in Tk/Tcl for drop-down dialog on ghetto_recorder package, first front-end
    global progress_master_percent   # separate for future us; go to single timer for each radio

    current_timer = 0
    while 1:

        if combo_master_timer:
            combo_time = (int(combo_master_timer) * 60 * 60)  # * 60
            percent = progress_bar_percent(current_timer, combo_time)
            if percent:
                progress_master_percent = percent
            else:
                progress_master_percent = 0
                current_timer = 0

            if percent >= 100:
                found = 0
                for _ in ghetto.GBase.dict_exit:
                    found += 1
                if found:
                    for recorder in ghetto.GBase.dict_exit:
                        ghetto.GBase.dict_exit[recorder] = True

        if not combo_master_timer:
            current_timer = 0
            progress_master_percent = 0

        current_timer += 1
        sleep(1)


def progress_bar_percent(current_timer, max_value):
    if not max_value:
        return False
    # doing some math, p = (P * 100) / G, percent = (math.percentage value * 100) / base
    cur_percent = round((current_timer * 100) / max_value, 4)  # 0,0001 for 24h reaction to show
    return cur_percent


def print_request_values(values):
    for val in values:
        print(' -- start print --')
        print(f'\tval in request.form.values(): {val}')
        print(f'\trequest.data {request.data}')
        print(f'\trequest.form {request.form}')
        print(f'\trequest.values {request.values}')
        print(f'\trequest.form.to_dict() {request.form.to_dict()}')
        print(' -- end print --')


def check_write_protected():
    try:
        download_dir = os.path.abspath(get_download_dir())
    except TypeError:
        return
    if download_dir is None:
        flash('Can not write to folder! No folder specified.', 'danger')
        return redirect(url_for('eisenhome_bp.index'))
    if download_dir:
        write_file = download_dir + '/eisen_write_test'
        try:
            with open(write_file, 'wb') as record_file:
                record_file.write(b'\x03')
            os.remove(write_file)
        except OSError:    # master of desaster
            flash('Can not write to folder!.' + download_dir, 'danger')
            return redirect(url_for('eisenhome_bp.index'))
    if not download_dir:
        flash('Can not write to folder! no folder specified' + download_dir, 'danger')
        return redirect(url_for('eisenhome_bp.index'))




import os
import webbrowser
from eisenradio.lib.eisdb import status_read_status_set


def get_env_snap():
    print('Eisenradio App runs in Ubuntu Snap Container, check environment:\n')
    print('SNAP_USER_COMMON (your Database lives here, backup if you like): ' + os.environ["SNAP_USER_COMMON"])
    print('SNAP_LIBRARY_PATH: ' + os.environ["SNAP_LIBRARY_PATH"])
    print('SNAP_COMMON: ' + os.environ["SNAP_COMMON"])
    print('SNAP_USER_DATA: ' + os.environ["SNAP_USER_DATA"])
    print('SNAP_DATA: ' + os.environ["SNAP_DATA"])
    print('SNAP_REVISION: ' + os.environ["SNAP_REVISION"])
    print('SNAP_NAME: ' + os.environ["SNAP_NAME"])
    print('SNAP_ARCH: ' + os.environ["SNAP_ARCH"])
    print('SNAP_VERSION: ' + os.environ["SNAP_VERSION"])
    print('SNAP: ' + os.environ["SNAP"])


def get_env_docker():
    print('\n\tEisenRadio App in Docker Container\n')
    print('\tUse url "http://localhost:5050/" to connect your Browser.\n')


def open_browser():
    print('\n\t---> Use url "http://localhost:5050/" to connect your Browser. <---\n')

    is_enabled = status_read_status_set(False, 'eisen_intern', 'browser_open', '1')
    if is_enabled:
        print(f' Browser auto start: Enabled\n')
        webbrowser.open('http://localhost:5050', new=2)
    if not is_enabled:
        print(f' Browser auto start: Disabled\n')


def main():
    is_snap_device = 'SNAP' in os.environ
    is_docker_device = 'DOCKER' in os.environ
    is_android_device = 'ANDROID_STORAGE' in os.environ

    if is_snap_device:
        get_env_snap()
    if is_docker_device:
        get_env_docker()
    if not is_android_device or not is_docker_device:
        open_browser()
